@@ -7,7 +7,17 @@ pub fn indent_if_start_of_line(items: PrintItems) -> Condition {
   let rc_path = items.into_rc_path();
   if_true_or(
     "indentIfStartOfLine",
-    |context| Some(condition_resolvers::is_start_of_line(context)),
+    condition_resolvers::is_start_of_line(),
+    ir_helpers::with_indent(rc_path.into()),
+    rc_path.into(),
+  )
+}
+
+pub fn indent_if_start_of_line_or_start_of_line_indented(items: PrintItems) -> Condition {
+  let rc_path = items.into_rc_path();
+  conditions::if_true_or(
+    "withIndentIfStartOfLineOrStartOfLineIndented",
+    condition_resolvers::is_start_of_line_or_is_start_of_line_indented(),
     ir_helpers::with_indent(rc_path.into()),
     rc_path.into(),
   )
@@ -17,48 +27,51 @@ pub fn with_indent_if_start_of_line_indented(items: PrintItems) -> Condition {
   let rc_path = items.into_rc_path();
   if_true_or(
     "withIndentIfStartOfLineIndented",
-    |context| Some(condition_resolvers::is_start_of_line_indented(context)),
+    condition_resolvers::is_start_of_line_indented(),
     ir_helpers::with_indent(rc_path.into()),
     rc_path.into(),
   )
 }
 
 pub struct NewLineIfHangingSpaceOtherwiseOptions {
-  pub start_info: Info,
-  pub end_info: Option<Info>,
+  pub start_lsil: LineStartIndentLevel,
+  pub end_lsil: Option<LineStartIndentLevel>,
   pub space_char: Option<PrintItems>,
 }
 
 pub fn new_line_if_hanging_space_otherwise(opts: NewLineIfHangingSpaceOtherwiseOptions) -> Condition {
   let space_char = opts.space_char.unwrap_or_else(|| " ".into());
-  let start_info = opts.start_info;
-  let end_info = opts.end_info;
+  let start_lsil = opts.start_lsil;
+  let end_lsil = opts.end_lsil;
 
   if_true_or(
     "newLineIfHangingSpaceOtherwise",
-    move |context| condition_resolvers::is_hanging(context, &start_info, &end_info),
+    Rc::new(move |context| condition_helpers::is_hanging(context, start_lsil, end_lsil)),
     Signal::NewLine.into(),
     space_char,
   )
 }
 
-pub fn new_line_if_hanging(start_info: Info, end_info: Option<Info>) -> Condition {
+pub fn new_line_if_hanging(start_lsil: LineStartIndentLevel, end_lsil: Option<LineStartIndentLevel>) -> Condition {
   if_true(
     "newlineIfHanging",
-    move |context| condition_resolvers::is_hanging(context, &start_info, &end_info),
+    Rc::new(move |context| condition_helpers::is_hanging(context, start_lsil, end_lsil)),
     Signal::NewLine.into(),
   )
 }
 
 /// This condition can be used to force the printer to jump back to the point
 /// this condition exists at once the provided info is resolved.
-pub fn force_reevaluation_once_resolved(info: Info) -> Condition {
+///
+/// NOTE: Don't use this. I'm going to remove it.
+pub fn force_reevaluation_once_resolved_deprecated(ln: LineNumber) -> Condition {
+  // note: it doesn't really matter what kind the info is (ex. LineNumber), but just that it's an info
   Condition::new(
-    "forceReevaluationOnceInfoResolved",
+    "forceReevaluationOnceoResolved",
     ConditionProperties {
       condition: Rc::new(move |context| {
-        let resolved_info = context.get_resolved_info(&info);
-        if resolved_info.is_some() {
+        let resolved_ln = context.resolved_line_number(ln);
+        if resolved_ln.is_some() {
           Some(false)
         } else {
           None
@@ -70,21 +83,21 @@ pub fn force_reevaluation_once_resolved(info: Info) -> Condition {
   )
 }
 
-pub fn new_line_if_multiple_lines_space_or_new_line_otherwise(start_info: Info, end_info: Option<Info>) -> Condition {
+pub fn new_line_if_multiple_lines_space_or_new_line_otherwise(start_ln: LineNumber, end_ln: Option<LineNumber>) -> Condition {
   if_true_or(
     "newLineIfMultipleLinesSpaceOrNewLineOtherwise",
-    move |context| {
-      let start_info = context.get_resolved_info(&start_info)?;
-      let end_info = {
-        if let Some(end_info) = &end_info {
-          context.get_resolved_info(end_info)?
+    Rc::new(move |context| {
+      let start_ln = context.resolved_line_number(start_ln)?;
+      let end_ln = {
+        if let Some(end_ln) = end_ln {
+          context.resolved_line_number(end_ln)?
         } else {
-          &context.writer_info
+          context.writer_info.line_number
         }
       };
 
-      Some(end_info.line_number > start_info.line_number)
-    },
+      Some(end_ln > start_ln)
+    }),
     Signal::NewLine.into(),
     Signal::SpaceOrNewLine.into(),
   )
@@ -93,7 +106,7 @@ pub fn new_line_if_multiple_lines_space_or_new_line_otherwise(start_info: Info,
 pub fn single_indent_if_start_of_line() -> Condition {
   if_true(
     "singleIndentIfStartOfLine",
-    |context| Some(condition_resolvers::is_start_of_line(context)),
+    condition_resolvers::is_start_of_line(),
     Signal::SingleIndent.into(),
   )
 }
@@ -112,7 +125,7 @@ pub fn if_above_width_or(width: u8, true_items: PrintItems, false_items: PrintIt
     ConditionProperties {
       condition: Rc::new(move |context| {
         let writer_info = &context.writer_info;
-        let first_indent_col = writer_info.line_start_column_number + (width as u32);
+        let first_indent_col = writer_info.line_start_column_number() + (width as u32);
         Some(writer_info.column_number > first_indent_col)
       }),
       true_path: Some(true_items),
@@ -121,40 +134,35 @@ pub fn if_above_width_or(width: u8, true_items: PrintItems, false_items: PrintIt
   )
 }
 
-pub fn if_true(name: &'static str, resolver: impl Fn(&mut ConditionResolverContext) -> Option<bool> + 'static, true_path: PrintItems) -> Condition {
+pub fn if_true(name: &'static str, resolver: ConditionResolver, true_path: PrintItems) -> Condition {
   Condition::new(
     name,
     ConditionProperties {
       true_path: Some(true_path),
       false_path: None,
-      condition: Rc::new(Box::new(resolver)),
+      condition: resolver,
     },
   )
 }
 
-pub fn if_true_or(
-  name: &'static str,
-  resolver: impl Fn(&mut ConditionResolverContext) -> Option<bool> + 'static,
-  true_path: PrintItems,
-  false_path: PrintItems,
-) -> Condition {
+pub fn if_true_or(name: &'static str, resolver: ConditionResolver, true_path: PrintItems, false_path: PrintItems) -> Condition {
   Condition::new(
     name,
     ConditionProperties {
       true_path: Some(true_path),
       false_path: Some(false_path),
-      condition: Rc::new(Box::new(resolver)),
+      condition: resolver,
     },
   )
 }
 
-pub fn if_false(name: &'static str, resolver: impl Fn(&mut ConditionResolverContext) -> Option<bool> + 'static, false_path: PrintItems) -> Condition {
+pub fn if_false(name: &'static str, resolver: ConditionResolver, false_path: PrintItems) -> Condition {
   Condition::new(
     name,
     ConditionProperties {
       true_path: None,
       false_path: Some(false_path),
-      condition: Rc::new(Box::new(resolver)),
+      condition: resolver,
     },
   )
 }
