@@ -2,9 +2,9 @@ use std::cell::UnsafeCell;
 use std::mem;
 use std::rc::Rc;
 
+use super::condition_resolvers;
 use super::printer::Printer;
-use super::utils::with_bump_allocator;
-use crate::formatting::id::IdCounter;
+use super::thread_state;
 
 /** Print Items */
 
@@ -15,8 +15,8 @@ pub struct PrintItems {
 }
 
 impl PrintItems {
-  pub fn new() -> PrintItems {
-    PrintItems {
+  pub fn new() -> Self {
+    Self {
       first_node: None,
       last_node: None,
     }
@@ -32,7 +32,7 @@ impl PrintItems {
 
   #[inline]
   fn push_item_internal(&mut self, item: PrintItem) {
-    let node = with_bump_allocator(|bump| {
+    let node = thread_state::with_bump_allocator(|bump| {
       let result = bump.alloc(PrintNodeCell::new(item));
       unsafe { std::mem::transmute::<&PrintNodeCell, UnsafePrintLifetime<PrintNodeCell>>(result) }
     });
@@ -70,7 +70,7 @@ impl PrintItems {
   }
 
   pub fn push_string(&mut self, item: String) {
-    let string_container = with_bump_allocator(|bump| {
+    let string_container = thread_state::with_bump_allocator(|bump| {
       let result = bump.alloc(StringContainer::new(item));
       unsafe { std::mem::transmute::<&StringContainer, UnsafePrintLifetime<StringContainer>>(result) }
     });
@@ -78,15 +78,28 @@ impl PrintItems {
   }
 
   pub fn push_condition(&mut self, condition: Condition) {
-    let condition = with_bump_allocator(|bump| {
+    let condition = thread_state::with_bump_allocator(|bump| {
       let result = bump.alloc(condition);
       unsafe { std::mem::transmute::<&Condition, UnsafePrintLifetime<Condition>>(result) }
     });
     self.push_item_internal(PrintItem::Condition(condition));
   }
 
-  pub fn push_info(&mut self, info: Info) {
-    self.push_item_internal(PrintItem::Info(info));
+  pub fn push_info(&mut self, info: impl Into<Info>) {
+    self.push_item_internal(PrintItem::Info(info.into()));
+  }
+
+  pub fn push_line_and_column(&mut self, line_and_col: LineAndColumn) {
+    self.push_info(line_and_col.line);
+    self.push_info(line_and_col.column);
+  }
+
+  pub fn push_anchor(&mut self, anchor: impl Into<Anchor>) {
+    self.push_item_internal(PrintItem::Anchor(anchor.into()));
+  }
+
+  pub fn push_reevaluation(&mut self, condition_reevaluation: ConditionReevaluation) {
+    self.push_item_internal(PrintItem::ConditionReevaluation(condition_reevaluation));
   }
 
   pub fn push_signal(&mut self, signal: Signal) {
@@ -121,7 +134,6 @@ impl PrintItems {
       for item in PrintItemsIterator::new(items) {
         match item {
           PrintItem::Signal(signal) => text.push_str(&get_line(format!("Signal::{:?}", signal), &indent_text)),
-          PrintItem::Info(info) => text.push_str(&get_line(format!("Info: {}", info.name), &indent_text)),
           PrintItem::Condition(condition) => {
             text.push_str(&get_line(format!("Condition: {}", condition.name), &indent_text));
             if let Some(true_path) = &condition.true_path {
@@ -135,6 +147,21 @@ impl PrintItems {
           }
           PrintItem::String(str_text) => text.push_str(&get_line(format!("`{}`", str_text.text), &indent_text)),
           PrintItem::RcPath(path) => text.push_str(&get_items_as_text(path, indent_text.clone())),
+          PrintItem::Anchor(Anchor::LineNumber(line_number_anchor)) => {
+            text.push_str(&get_line(format!("Line number anchor: {}", line_number_anchor.name()), &indent_text))
+          }
+          PrintItem::Info(info) => {
+            let (desc, name) = match info {
+              Info::LineNumber(info) => ("Line number", info.name()),
+              Info::ColumnNumber(info) => ("Column number", info.name()),
+              Info::IsStartOfLine(info) => ("Is start of line", info.name()),
+              Info::IndentLevel(info) => ("Indent level", info.name()),
+              Info::LineStartColumnNumber(info) => ("Line start column number", info.name()),
+              Info::LineStartIndentLevel(info) => ("Line start indent level", info.name()),
+            };
+            text.push_str(&get_line(format!("{}: {}", desc, name), &indent_text))
+          }
+          PrintItem::ConditionReevaluation(reevaluation) => text.push_str(&get_line(format!("Condition reevaluation: {}", reevaluation.name()), &indent_text)),
         }
       }
 
@@ -156,8 +183,8 @@ pub struct PrintItemsIterator {
 }
 
 impl PrintItemsIterator {
-  pub fn new(path: PrintItemPath) -> PrintItemsIterator {
-    PrintItemsIterator { node: Some(path) }
+  pub fn new(path: PrintItemPath) -> Self {
+    Self { node: Some(path) }
   }
 }
 
@@ -234,18 +261,18 @@ where
 pub struct Trace {
   /// The relative time of the trace from the start of printing in nanoseconds.
   pub nanos: u128,
-  pub print_node_id: usize,
+  pub print_node_id: u32,
   #[serde(skip_serializing_if = "Option::is_none")]
-  pub writer_node_id: Option<usize>,
+  pub writer_node_id: Option<u32>,
 }
 
 #[cfg(feature = "tracing")]
 #[derive(serde::Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct TraceWriterNode {
-  pub writer_node_id: usize,
+  pub writer_node_id: u32,
   #[serde(skip_serializing_if = "Option::is_none")]
-  pub previous_node_id: Option<usize>,
+  pub previous_node_id: Option<u32>,
   pub text: String,
 }
 
@@ -253,9 +280,9 @@ pub struct TraceWriterNode {
 #[derive(serde::Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct TracePrintNode {
-  pub print_node_id: usize,
+  pub print_node_id: u32,
   #[serde(skip_serializing_if = "Option::is_none")]
-  pub next_print_node_id: Option<usize>,
+  pub next_print_node_id: Option<u32>,
   pub print_item: TracePrintItem,
 }
 
@@ -268,14 +295,54 @@ pub enum TracePrintItem {
   Info(TraceInfo),
   Signal(Signal),
   /// Identifier to the print node.
-  RcPath(usize),
+  RcPath(u32),
+  Anchor(TraceLineNumberAnchor),
+  ConditionReevaluation(TraceConditionReevaluation),
+}
+
+#[cfg(feature = "tracing")]
+#[derive(serde::Serialize)]
+#[serde(tag = "kind", content = "content", rename_all = "camelCase")]
+pub enum TraceInfo {
+  LineNumber(TraceInfoInner),
+  ColumnNumber(TraceInfoInner),
+  IsStartOfLine(TraceInfoInner),
+  IndentLevel(TraceInfoInner),
+  LineStartColumnNumber(TraceInfoInner),
+  LineStartIndentLevel(TraceInfoInner),
 }
 
 #[cfg(feature = "tracing")]
 #[derive(serde::Serialize)]
 #[serde(rename_all = "camelCase")]
-pub struct TraceInfo {
-  pub info_id: usize,
+pub struct TraceInfoInner {
+  pub info_id: u32,
+  pub name: String,
+}
+
+#[cfg(feature = "tracing")]
+impl TraceInfoInner {
+  pub fn new(info_id: u32, name: &str) -> Self {
+    Self {
+      info_id,
+      name: name.to_string(),
+    }
+  }
+}
+
+#[cfg(feature = "tracing")]
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub struct TraceLineNumberAnchor {
+  pub anchor_id: u32,
+  pub name: String,
+}
+
+#[cfg(feature = "tracing")]
+#[derive(serde::Serialize)]
+#[serde(rename_all = "camelCase")]
+pub struct TraceConditionReevaluation {
+  pub condition_id: u32,
   pub name: String,
 }
 
@@ -283,19 +350,16 @@ pub struct TraceInfo {
 #[derive(serde::Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct TraceCondition {
-  pub condition_id: usize,
+  pub condition_id: u32,
   pub name: String,
   pub is_stored: bool,
+  pub store_save_point: bool,
   #[serde(skip_serializing_if = "Option::is_none")]
   /// Identifier to the true path print node.
-  pub true_path: Option<usize>,
+  pub true_path: Option<u32>,
   #[serde(skip_serializing_if = "Option::is_none")]
   /// Identifier to the false path print node.
-  pub false_path: Option<usize>,
-  #[serde(skip_serializing_if = "Option::is_none")]
-  /// Any infos that should cause the re-evaluation of this condition.
-  /// This is only done on request for performance reasons.
-  pub dependent_infos: Option<Vec<usize>>,
+  pub false_path: Option<u32>,
 }
 
 /** Print Node */
@@ -304,12 +368,7 @@ pub struct PrintNode {
   pub(super) next: Option<PrintItemPath>,
   pub(super) item: PrintItem,
   #[cfg(feature = "tracing")]
-  pub print_node_id: usize,
-}
-
-thread_local! {
-  #[cfg(feature = "tracing")]
-  static PRINT_NODE_IDS: IdCounter = IdCounter::default();
+  pub print_node_id: u32,
 }
 
 impl PrintNode {
@@ -318,7 +377,7 @@ impl PrintNode {
       item,
       next: None,
       #[cfg(feature = "tracing")]
-      print_node_id: IdCounter::next(&PRINT_NODE_IDS),
+      print_node_id: thread_state::next_print_node_id(),
     }
   }
 
@@ -379,7 +438,7 @@ impl PrintNodeCell {
   }
 
   #[cfg(feature = "tracing")]
-  pub(super) fn get_node_id(&self) -> usize {
+  pub(super) fn get_node_id(&self) -> u32 {
     unsafe { (*self.get_node()).print_node_id }
   }
 
@@ -415,9 +474,11 @@ type UnsafePrintLifetime<T> = &'static T;
 pub enum PrintItem {
   String(UnsafePrintLifetime<StringContainer>),
   Condition(UnsafePrintLifetime<Condition>),
-  Info(Info),
   Signal(Signal),
   RcPath(PrintItemPath),
+  Anchor(Anchor),
+  Info(Info),
+  ConditionReevaluation(ConditionReevaluation),
 }
 
 #[derive(Clone, PartialEq, Copy, Debug, serde::Serialize)]
@@ -459,42 +520,320 @@ pub enum Signal {
   SpaceIfNotTrailing,
 }
 
-/// Can be used to get information at a certain location being printed. These
-/// can be resolved by providing the info object to a condition context's
-/// get_resolved_info(&info) method.
+#[derive(Clone)]
+pub enum Anchor {
+  LineNumber(LineNumberAnchor),
+}
+
+impl From<LineNumberAnchor> for Anchor {
+  fn from(anchor: LineNumberAnchor) -> Self {
+    Anchor::LineNumber(anchor)
+  }
+}
+
+/// Handles updating the position of a future resolved line
+/// number if the anchor changes.
+#[derive(Clone)]
+pub struct LineNumberAnchor {
+  id: u32,
+  line_number: LineNumber,
+}
+
+impl LineNumberAnchor {
+  pub fn new(line_number: LineNumber) -> Self {
+    Self {
+      id: thread_state::next_line_number_anchor_id(),
+      line_number,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn line_number_id(&self) -> u32 {
+    self.line_number.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    self.line_number.name()
+  }
+}
+
 #[derive(Clone, PartialEq, Copy, Debug)]
-pub struct Info {
-  /// Unique identifier.
-  id: usize,
+pub enum Info {
+  LineNumber(LineNumber),
+  ColumnNumber(ColumnNumber),
+  IsStartOfLine(IsStartOfLine),
+  IndentLevel(IndentLevel),
+  LineStartColumnNumber(LineStartColumnNumber),
+  LineStartIndentLevel(LineStartIndentLevel),
+}
+
+impl From<LineNumber> for Info {
+  fn from(info: LineNumber) -> Self {
+    Info::LineNumber(info)
+  }
+}
+
+impl From<ColumnNumber> for Info {
+  fn from(info: ColumnNumber) -> Self {
+    Info::ColumnNumber(info)
+  }
+}
+
+impl From<IsStartOfLine> for Info {
+  fn from(info: IsStartOfLine) -> Self {
+    Info::IsStartOfLine(info)
+  }
+}
+
+impl From<IndentLevel> for Info {
+  fn from(info: IndentLevel) -> Self {
+    Info::IndentLevel(info)
+  }
+}
+
+impl From<LineStartColumnNumber> for Info {
+  fn from(info: LineStartColumnNumber) -> Self {
+    Info::LineStartColumnNumber(info)
+  }
+}
+
+impl From<LineStartIndentLevel> for Info {
+  fn from(info: LineStartIndentLevel) -> Self {
+    Info::LineStartIndentLevel(info)
+  }
+}
+
+/// Helper IR that holds line and column number IR.
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct LineAndColumn {
+  pub line: LineNumber,
+  pub column: ColumnNumber,
+}
+
+impl LineAndColumn {
+  pub fn new(name: &'static str) -> Self {
+    Self {
+      line: LineNumber::new(name),
+      column: ColumnNumber::new(name),
+    }
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct LineNumber {
+  id: u32,
   /// Name for debugging purposes.
   #[cfg(debug_assertions)]
   name: &'static str,
 }
 
-thread_local! {
-  static INFO_IDS: IdCounter = IdCounter::default();
-}
-
-impl Info {
-  pub fn new(_name: &'static str) -> Info {
-    Info {
-      id: IdCounter::next(&INFO_IDS),
+impl LineNumber {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_line_number_id(),
       #[cfg(debug_assertions)]
       name: _name,
     }
   }
 
   #[inline]
-  pub fn get_unique_id(&self) -> usize {
+  pub fn unique_id(&self) -> u32 {
     self.id
   }
 
   #[inline]
-  pub fn get_name(&self) -> &'static str {
+  pub fn name(&self) -> &'static str {
     #[cfg(debug_assertions)]
     return self.name;
     #[cfg(not(debug_assertions))]
-    return "info";
+    return "line_number";
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct ColumnNumber {
+  id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl ColumnNumber {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_column_number_id(),
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "column_number";
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct IsStartOfLine {
+  id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl IsStartOfLine {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_is_start_of_line_id(),
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "is_start_of_line";
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct LineStartColumnNumber {
+  id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl LineStartColumnNumber {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_line_start_column_number_id(),
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "line_start_column_number";
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct IndentLevel {
+  id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl IndentLevel {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_indent_level_id(),
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "indent_level";
+  }
+}
+
+#[derive(Clone, PartialEq, Copy, Debug)]
+pub struct LineStartIndentLevel {
+  id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl LineStartIndentLevel {
+  pub fn new(_name: &'static str) -> Self {
+    Self {
+      id: thread_state::next_line_start_indent_level_id(),
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  #[inline]
+  pub fn unique_id(&self) -> u32 {
+    self.id
+  }
+
+  #[inline]
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "line_start_indent_level";
+  }
+}
+
+/// Used to re-evaluate a condition.
+#[derive(Clone, Copy, PartialEq, Debug)]
+pub struct ConditionReevaluation {
+  pub(crate) condition_id: u32,
+  /// Name for debugging purposes.
+  #[cfg(debug_assertions)]
+  name: &'static str,
+}
+
+impl ConditionReevaluation {
+  pub(crate) fn new(_name: &'static str, condition_id: u32) -> Self {
+    ConditionReevaluation {
+      condition_id,
+      #[cfg(debug_assertions)]
+      name: _name,
+    }
+  }
+
+  pub fn name(&self) -> &'static str {
+    #[cfg(debug_assertions)]
+    return self.name;
+    #[cfg(not(debug_assertions))]
+    return "condition_reevaluation";
   }
 }
 
@@ -505,81 +844,69 @@ impl Info {
 #[derive(Clone)]
 pub struct Condition {
   /// Unique identifier.
-  id: usize,
+  id: u32,
   /// Name for debugging purposes.
   #[cfg(debug_assertions)]
   name: &'static str,
-  /// If a reference has been created for the condition via `get_reference()`. If so, the printer
+  /// If a reference has been created for the condition via `create_reference()`. If so, the printer
   /// will store the condition and it will be retrievable via a condition resolver.
   pub(super) is_stored: bool,
+  pub(super) store_save_point: bool,
   /// The condition to resolve.
-  pub(super) condition: Rc<ConditionResolver>,
+  pub(super) condition: ConditionResolver,
   /// The items to print when the condition is true.
   pub(super) true_path: Option<PrintItemPath>,
   /// The items to print when the condition is false or undefined (not yet resolved).
   pub(super) false_path: Option<PrintItemPath>,
-  /// Any infos that should cause the re-evaluation of this condition.
-  /// This is only done on request for performance reasons.
-  pub(super) dependent_infos: Option<Vec<Info>>,
-}
-
-thread_local! {
-  static CONDITION_IDS: IdCounter = IdCounter::default();
 }
 
 impl Condition {
-  pub fn new(name: &'static str, properties: ConditionProperties) -> Condition {
-    Condition::new_internal(name, properties, None)
+  pub fn new(name: &'static str, properties: ConditionProperties) -> Self {
+    Self::new_internal(name, properties)
   }
 
-  pub fn new_true() -> Condition {
-    Condition::new_internal(
+  pub fn new_true() -> Self {
+    Self::new_internal(
       "trueCondition",
       ConditionProperties {
-        condition: Rc::new(|_| Some(true)),
+        condition: condition_resolvers::true_resolver(),
         true_path: None,
         false_path: None,
       },
-      None,
     )
   }
 
-  pub fn new_false() -> Condition {
-    Condition::new_internal(
+  pub fn new_false() -> Self {
+    Self::new_internal(
       "falseCondition",
       ConditionProperties {
-        condition: Rc::new(|_| Some(false)),
+        condition: condition_resolvers::false_resolver(),
         true_path: None,
         false_path: None,
       },
-      None,
     )
   }
 
-  pub fn new_with_dependent_infos(name: &'static str, properties: ConditionProperties, dependent_infos: Vec<Info>) -> Condition {
-    Condition::new_internal(name, properties, Some(dependent_infos))
-  }
-
-  fn new_internal(_name: &'static str, properties: ConditionProperties, dependent_infos: Option<Vec<Info>>) -> Condition {
-    Condition {
-      id: IdCounter::next(&CONDITION_IDS),
-      is_stored: dependent_infos.is_some(),
+  fn new_internal(_name: &'static str, properties: ConditionProperties) -> Self {
+    Self {
+      id: thread_state::next_condition_id(),
+      is_stored: false,
+      store_save_point: false,
       #[cfg(debug_assertions)]
       name: _name,
       condition: properties.condition,
       true_path: properties.true_path.and_then(|x| x.first_node),
       false_path: properties.false_path.and_then(|x| x.first_node),
-      dependent_infos,
     }
   }
 
   #[inline]
-  pub fn get_unique_id(&self) -> usize {
+  pub fn unique_id(&self) -> u32 {
     self.id
   }
 
   #[inline]
-  pub fn get_name(&self) -> &'static str {
+  pub fn name(&self) -> &'static str {
     #[cfg(debug_assertions)]
     return self.name;
     #[cfg(not(debug_assertions))]
@@ -587,12 +914,12 @@ impl Condition {
   }
 
   #[inline]
-  pub fn get_true_path(&self) -> &Option<PrintItemPath> {
+  pub fn true_path(&self) -> &Option<PrintItemPath> {
     &self.true_path
   }
 
   #[inline]
-  pub fn get_false_path(&self) -> &Option<PrintItemPath> {
+  pub fn false_path(&self) -> &Option<PrintItemPath> {
     &self.false_path
   }
 
@@ -601,9 +928,15 @@ impl Condition {
     (self.condition)(context)
   }
 
-  pub fn get_reference(&mut self) -> ConditionReference {
+  pub fn create_reference(&mut self) -> ConditionReference {
     self.is_stored = true;
-    ConditionReference::new(self.get_name(), self.id)
+    ConditionReference::new(self.name(), self.id)
+  }
+
+  pub fn create_reevaluation(&mut self) -> ConditionReevaluation {
+    self.store_save_point = true;
+    self.is_stored = true;
+    ConditionReevaluation::new(self.name(), self.id)
   }
 }
 
@@ -611,11 +944,11 @@ impl Condition {
 pub struct ConditionReference {
   #[cfg(debug_assertions)]
   pub(super) name: &'static str,
-  pub(super) id: usize,
+  pub(super) id: u32,
 }
 
 impl ConditionReference {
-  pub(super) fn new(_name: &'static str, id: usize) -> ConditionReference {
+  pub(super) fn new(_name: &'static str, id: u32) -> ConditionReference {
     ConditionReference {
       #[cfg(debug_assertions)]
       name: _name,
@@ -624,7 +957,7 @@ impl ConditionReference {
   }
 
   #[inline]
-  pub(super) fn get_name(&self) -> &'static str {
+  pub(super) fn name(&self) -> &'static str {
     #[cfg(debug_assertions)]
     return self.name;
     #[cfg(not(debug_assertions))]
@@ -632,16 +965,16 @@ impl ConditionReference {
   }
 
   /// Creates a condition resolver that checks the value of the condition this references.
-  pub fn create_resolver(&self) -> impl Fn(&mut ConditionResolverContext) -> Option<bool> + Clone + 'static {
+  pub fn create_resolver(&self) -> ConditionResolver {
     let captured_self = *self;
-    move |condition_context: &mut ConditionResolverContext| condition_context.get_resolved_condition(&captured_self)
+    Rc::new(move |condition_context: &mut ConditionResolverContext| condition_context.resolved_condition(&captured_self))
   }
 }
 
 /// Properties for the condition.
 pub struct ConditionProperties {
   /// The condition to resolve.
-  pub condition: Rc<ConditionResolver>,
+  pub condition: ConditionResolver,
   /// The items to print when the condition is true.
   pub true_path: Option<PrintItems>,
   /// The items to print when the condition is false or undefined (not yet resolved).
@@ -649,7 +982,7 @@ pub struct ConditionProperties {
 }
 
 /// Function used to resolve a condition.
-pub type ConditionResolver = dyn Fn(&mut ConditionResolverContext) -> Option<bool>;
+pub type ConditionResolver = Rc<dyn Fn(&mut ConditionResolverContext) -> Option<bool>>;
 
 /// Context used when resolving a condition.
 pub struct ConditionResolverContext<'a, 'b> {
@@ -663,26 +996,58 @@ impl<'a, 'b> ConditionResolverContext<'a, 'b> {
     ConditionResolverContext { printer, writer_info }
   }
 
-  /// Gets if a condition was true, false, or returns undefined when not yet resolved.
-  /// A condition reference can be retrieved by calling the `get_reference()` on a condition.
-  pub fn get_resolved_condition(&mut self, condition_reference: &ConditionReference) -> Option<bool> {
-    self.printer.get_resolved_condition(condition_reference)
+  /// Gets if a condition was true, false, or returns None when not yet resolved.
+  /// A condition reference can be retrieved by calling the `create_reference()` on a condition.
+  pub fn resolved_condition(&mut self, condition_reference: &ConditionReference) -> Option<bool> {
+    self.printer.resolved_condition(condition_reference)
   }
 
-  /// Gets the writer info at a specified info or returns undefined when not yet resolved.
-  pub fn get_resolved_info(&self, info: &Info) -> Option<&WriterInfo> {
-    self.printer.get_resolved_info(info)
+  /// Gets a resolved line and column.
+  pub fn resolved_line_and_column(&mut self, line_and_column: LineAndColumn) -> Option<(u32, u32)> {
+    let line = self.printer.resolved_line_number(line_and_column.line)?;
+    let column = self.printer.resolved_column_number(line_and_column.column)?;
+    Some((line, column))
   }
 
-  /// Clears the info result from being stored.
-  pub fn clear_info(&mut self, info: &Info) {
-    self.printer.clear_info(info)
+  /// Gets the line number or returns None when not yet resolved.
+  pub fn resolved_line_number(&mut self, line_number: LineNumber) -> Option<u32> {
+    self.printer.resolved_line_number(line_number)
   }
 
-  /// Gets if the provided info has moved positions since the last check.
-  /// Returns None when the info can't be resolved. Returns Some(false) the first time this is called.
-  pub fn has_info_moved(&mut self, info: &Info) -> Option<bool> {
-    self.printer.has_info_moved(info)
+  /// Gets the column number or returns None when not yet resolved.
+  pub fn resolved_column_number(&mut self, column_number: ColumnNumber) -> Option<u32> {
+    self.printer.resolved_column_number(column_number)
+  }
+
+  /// Gets if the info is at the start of the line or returns None when not yet resolved.
+  pub fn resolved_is_start_of_line(&mut self, is_start_of_line: IsStartOfLine) -> Option<bool> {
+    self.printer.resolved_is_start_of_line(is_start_of_line)
+  }
+
+  /// Gets if the indent level at this info or returns None when not yet resolved.
+  pub fn resolved_indent_level(&mut self, indent_level: IndentLevel) -> Option<u8> {
+    self.printer.resolved_indent_level(indent_level)
+  }
+
+  /// Gets the column number at the start of the line this info appears or returns None when not yet resolved.
+  pub fn resolved_line_start_column_number(&mut self, line_start_column_number: LineStartColumnNumber) -> Option<u32> {
+    self.printer.resolved_line_start_column_number(line_start_column_number)
+  }
+
+  /// Gets the indent level at the start of the line this info appears or returns None when not yet resolved.
+  pub fn resolved_line_start_indent_level(&mut self, line_start_indent_level: LineStartIndentLevel) -> Option<u8> {
+    self.printer.resolved_line_start_indent_level(line_start_indent_level)
+  }
+
+  /// Clears the line and column from being stored.
+  pub fn clear_line_and_column(&mut self, lc: LineAndColumn) {
+    self.clear_info(lc.line);
+    self.clear_info(lc.column);
+  }
+
+  /// Clears the info from being stored.
+  pub fn clear_info(&mut self, info: impl Into<Info>) {
+    self.printer.clear_info(info.into())
   }
 }
 
@@ -698,9 +1063,9 @@ pub struct StringContainer {
 
 impl StringContainer {
   /// Creates a new string container.
-  pub fn new(text: String) -> StringContainer {
+  pub fn new(text: String) -> Self {
     let char_count = text.chars().count() as u32;
-    StringContainer { text, char_count }
+    Self { text, char_count }
   }
 }
 
@@ -711,17 +1076,33 @@ pub struct WriterInfo {
   pub column_number: u32,
   pub indent_level: u8,
   pub line_start_indent_level: u8,
-  pub line_start_column_number: u32,
+  pub indent_width: u8,
+  pub expect_newline_next: bool,
 }
 
 impl WriterInfo {
-  /// Gets if the current column number equals the line start column number.
+  /// Gets if the current column number equals the line start column number
+  /// or if a newline is expected next.
   pub fn is_start_of_line(&self) -> bool {
-    self.column_number == self.line_start_column_number
+    self.expect_newline_next || self.is_column_number_at_line_start()
+  }
+
+  /// Gets if the start of the line is indented.
+  pub fn is_start_of_line_indented(&self) -> bool {
+    self.line_start_indent_level > self.indent_level
+  }
+
+  /// Gets if the current column number is at the line start column number.
+  pub fn is_column_number_at_line_start(&self) -> bool {
+    self.column_number == self.line_start_column_number()
+  }
+
+  pub fn line_start_column_number(&self) -> u32 {
+    (self.line_start_indent_level as u32) * (self.indent_width as u32)
   }
 
   /// Gets the line and column number.
-  pub fn get_line_and_column(&self) -> (u32, u32) {
+  pub fn line_and_column(&self) -> (u32, u32) {
     (self.line_number, self.column_number)
   }
 }
